// ----------------------------------------------------------------
// The contents of this file are distributed under the CC0 license.
// See http://creativecommons.org/publicdomain/zero/1.0/
// ----------------------------------------------------------------

import javax.swing.JFrame;
import java.io.File;


class Sim extends JFrame
{
	volatile boolean running;
	Thread pump = null;
	Model model;
	View view;
	Controller controller;

	public Sim(double trans_noise_dev, double obs_noise_dev) throws Exception
	{
		NeuralNet obs = new NeuralNet(Json.load("obs.json"));
		NeuralNet trans = new NeuralNet(Json.load("trans.json"));

		model = new Model(obs, trans);
		view = new View(model, obs_noise_dev);
		controller = new Controller(model, view, trans_noise_dev);
		view.addMouseListener(controller);
		addKeyListener(controller);
		this.setTitle("Crane Simulator");
		this.setSize(500, 500);
		this.getContentPane().add(view);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setVisible(true);
		running = true;
	}

	public void run()
	{
		while(running)
		{
			//long time = System.currentTimeMillis();
			controller.update();
			model.update();
			if(view.visualizing)
			{
				repaint(); // Indirectly calls View.paintComponent
				try {
					Thread.sleep(30);
				} catch(Exception e) {
					e.printStackTrace();
					System.exit(1);
				}
			}
		}
	}

	public static void main(String[] args) throws Exception
	{
		if(args.length < 2)
		{
			System.out.println("Usage:   java Sim [trans_noise_dev] [obs_noise_dev]");
			System.out.println("Example: java Sim 0.0 0.0");
			return;
		}
		double trans_noise_dev = Double.parseDouble(args[0]);
		double obs_noise_dev = Double.parseDouble(args[1]);
		Sim s = new Sim(trans_noise_dev, obs_noise_dev);
		s.run();
	}
}
