// ----------------------------------------------------------------
// The contents of this file are distributed under the CC0 license.
// See http://creativecommons.org/publicdomain/zero/1.0/
// ----------------------------------------------------------------

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Image;
import javax.imageio.ImageIO;
import java.io.IOException;
import java.awt.Graphics;
import java.io.File;
import java.awt.Color;

class View extends JPanel
{
	static final int GRID_SCALE = 20;

	Model model;
	volatile boolean visualizing;
	double obs_dev;
	double[] in;

	View(Model m, double obs_noise_dev)
	{
		visualizing = true;
		model = m;
		obs_dev = obs_noise_dev;
		in = new double[4];
	}

	static int state_to_screen_x(double x)
	{
		int v = 100 + (int)(GRID_SCALE * (10.0 * x + 10));
		return v;
	}

	static int state_to_screen_y(double y)
	{
		int v = 20 * GRID_SCALE - (int)(GRID_SCALE * (10.0 * y + 10));
		return v;
	}

	public void paintComponent(Graphics g)
	{
		// Draw the predicted observation
		for(int y = 0; y < 48; y++)
		{
			for(int x = 0; x < 64; x++)
			{
				in[0] = (double)x / 64;
				in[1] = (double)y / 48;
				in[2] = model.state[0];
				in[3] = model.state[1];
				double[] pred = model.obs.forwardProp(in);
				int rr = Math.max(0, Math.min(255, (int)((pred[0] + model.rand.nextGaussian() * obs_dev) * 256)));
				int gg = Math.max(0, Math.min(255, (int)((pred[1] + model.rand.nextGaussian() * obs_dev) * 256)));
				int bb = Math.max(0, Math.min(255, (int)((pred[2] + model.rand.nextGaussian() * obs_dev) * 256)));
				g.setColor(new Color(rr, gg, bb));
				g.drawLine(x, y, x, y); // todo: this is an expensive way to set a pixel
			}
		}

		// Draw a grid
		g.setColor(new Color(0xb0, 0xb0, 0xb0));
		for(int i = 0; i <= 20; i++)
		{
			int x = 100 + GRID_SCALE * i;
			int y = GRID_SCALE * (20 - i);
			g.drawLine(100, y, 100 + 20 * GRID_SCALE, y); // horizontal line
			g.drawLine(x, 0, x, 20 * GRID_SCALE); // vertical line
		}

		// Draw the state
		g.setColor(new Color(0, 0, 0x80));
		g.fillRect(state_to_screen_x(model.state[0]) - 2, state_to_screen_y(model.state[1]) - 2, 5, 5);

	
	
	
/*
		// Draw the ground and boundaries
		int left = width / 2 - (int)(model.boundary * scale) - cart_width / 2;
		int right = width / 2 + (int)(model.boundary * scale) + cart_width / 2;
		g.drawLine(left, ground_y, right, ground_y);
		g.drawLine(left, ground_y, left, ground_y - 30);
		g.drawLine(right, ground_y, right, ground_y - 30);
		
		// Draw the cart
		int pole_x = width / 2 + (int)(scale * model.cart_position);
		int cart_x = pole_x - cart_width / 2;
		int cart_y = ground_y - cart_wheel_diameter - cart_height;
		g.drawRect(cart_x, cart_y, cart_width, cart_height);
		g.drawOval(cart_x, cart_y + cart_height, cart_wheel_diameter, cart_wheel_diameter);
		g.drawOval(cart_x + cart_width - cart_wheel_diameter, cart_y + cart_height, cart_wheel_diameter, cart_wheel_diameter);

		// Draw the pole
		double horiz_reach = scale * Math.cos(model.pole_angle + Math.PI / 2);
		double vert_reach = scale * Math.sin(model.pole_angle + Math.PI / 2);
		g.drawLine(pole_x, cart_y, pole_x + (int)horiz_reach, cart_y + (int)vert_reach);
*/
	}
}
